#!/usr/bin/perl -w
#
# bogohelper.pl
# Copyright (C) 2006 Juan J. Martínez <reidrac@blackshell.usebox.net>
# Some rights reserved, read LICENSE
#

use strict;

use Getopt::Std;
use POSIX qw(setuid getuid);
use Email::MIME;

our(%opts);

my $VERSION="0.1";

# some default values
my $DEF_USER="bogofilter";
my $DEF_BOGOFILTER="/usr/local/bin/bogofilter";
# 1 only ham, 2 only spam, 3 both
my $DEF_OPERATION=3;

sub usage
{
	print <<ENDUSAGE
bogohelper.pl $VERSION
Copyright (C) 2006 Juan J. Martínez <reidrac\@blackshell.usebox.net>
Some rights reserved, read LICENSE.

bogohelper.pl -h
bogohelper.pl [-u user] [-N | -S] [-f from] [-b bogofilter]

-h             This help
-u user        User running bogofilter (default: bogofilter)
-N             Only correct ham
-S             Only correct spam
-f regf        Validate requests when regex 'regf' matches 'From:' header
-b bogofilter  Path to bogofilter binary (default: /usr/local/bin/bogofilter)

ENDUSAGE

}

# MAIN ------------------

getopts('hu:NSf:b:', \%opts);

if(defined($opts{h}))
{
	usage;
	exit 0;
}

if(defined($opts{N}) and defined($opts{S}))
{
	printf STDERR "ERROR: -N and -S are incompatible";
	exit 1;
}

# which kind of errors should we fix?
my $correct=$DEF_OPERATION;
$correct=1 if defined($opts{'N'});
$correct=2 if defined($opts{'S'});

# drop privileges (from aliases sendmail may run bogohelper as root, else
# install the script suid)
my $uid=getpwnam(defined($opts{u}) ? $opts{u} : $DEF_USER);

if(!$uid)
{
	printf STDERR "ERROR: getpwnam failed\n";
	exit 1;
}

if(!setuid($uid))
{
	printf STDERR "ERROR: setuid failed (current ".getuid().")\n";
	exit 1;
}

my $bogofilter=defined($opts{b}) ? $opts{b} : $DEF_BOGOFILTER;

if(! -x $bogofilter)
{
	print STDERR "ERROR: bogofilter binary ($bogofilter) not found or not executable\n";
	exit 1;
}

my $message;
{
	local $/;
	$message=<>;
}

my $parsed=Email::MIME->new($message);
my @parts=$parsed->parts;

# at least two parts baby
exit 0 if scalar(@parts) < 2;

# filter from "From" header
if(defined($opts{f}))
{
	exit 0 unless $parsed->header("From") =~ m/$opts{f}/;
}

# ignore carp warnings from Email::MIME::ContentType
close STDERR;

# look for suitable attachments to correct
foreach my $p (@parts)
{
	if($p->content_type eq "message/rfc822")
	{
		my $attach=Email::MIME->new($p->body_raw);
		my $spam=($attach->header("X-Bogosity") =~ m/(Yes, tests=bogofilter|Spam, spamicity=)/);
		my $ham=($attach->header("X-Bogosity") =~ m/(No, tests=bogofilter|Ham, spamicity=)/);

		# check: there must be a bogofilter classification
		next if $spam eq $ham;

		# it must be an allowed correction
		next if $ham and not ($correct & 1);
		next if $spam and not ($correct & 2);

		# ok, do it
		my $cl;

		$cl="$bogofilter -l -Sn" if $spam eq 1;
		$cl="$bogofilter -l -Ns" if $ham eq 1;

		delete $ENV{PATH};
		open(PROC,"| $cl") or
			die("Unable to exec bogofilter: $!");
		print PROC $p->body_raw;
		close PROC;
	}
}

# EOF
